import { Attribute, BrandingOverlayType } from '@hyperledger/aries-oca/build/legacy';
import { useNavigation } from '@react-navigation/native';
import startCase from 'lodash.startcase';
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { useTranslation } from 'react-i18next';
import { FlatList, Image, ImageBackground, Linking, Text, View, TouchableOpacity } from 'react-native';
import Icon from 'react-native-vector-icons/MaterialIcons';
import { TOKENS, useServices } from '../../container-api';
import { useTheme } from '../../contexts/theme';
import { credentialTextColor, getCredentialIdentifiers, toImageSource } from '../../utils/credential';
import { formatIfDate, useCredentialConnectionLabel, isDataUrl, pTypeToText, getSecondaryBackgroundColor } from '../../utils/helpers';
import { shadeIsLightOrDark, Shade } from '../../utils/luminance';
import { testIdWithKey } from '../../utils/testable';
import CardWatermark from './CardWatermark';
import CredentialActionFooter from './CredentialCard11ActionFooter';
import CredentialCard11Logo from './CredentialCard11Logo';
import useCredentialCardStyles from '../../hooks/credential-card-styles';
import CredentialIssuerBody from './CredentialCard11Issuer';
export let CredentialErrors = /*#__PURE__*/function (CredentialErrors) {
  CredentialErrors[CredentialErrors["Revoked"] = 0] = "Revoked";
  CredentialErrors[CredentialErrors["NotInWallet"] = 1] = "NotInWallet";
  CredentialErrors[CredentialErrors["PredicateError"] = 2] = "PredicateError";
  return CredentialErrors;
}({}); // Credential requested for proof contains a predicate match that is not satisfied
/*
  A card is defined as a nx8 (height/rows x width/columns) grid.
  | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |
  | 2 |   |   |   |   |   |   |   |
  | 3 |   |   |   |   |   |   |   |
  | 4 |   |   |   |   |   |   |   |
  ...

  The card width is the full screen width.

  Secondary Body (1):
  Primary Body   (2): Small Logo (1x1) -> L (shifted left by 50%)
                      Issuer Name (1x6)
                      Credential Name (1x6)
                      Primary Attribute 1 (1x6)
                      Primary Attribute 2 (1x6)
  Status         (3): Icon (1x1) -> S

   (1)            (2)           (3)
  | L | Issuer Name            | S |
  |   | Credential Name        |   |
  |   |                        |   |
  |   | Primary Attribute 1    |   |
  |   | Primary Attribute 2    |   |
  ...

  Note: The small logo MUST be provided as 1x1 (height/width) ratio.
 */

const CredentialCard11 = ({
  credential,
  style = {},
  displayItems,
  onPress = undefined,
  elevated = false,
  credName,
  credDefId,
  schemaId,
  proof,
  hasAltCredentials,
  credentialErrors = [],
  handleAltCredChange,
  brandingOverlay,
  hideSlice = false,
  extraOverlayParameter,
  brandingOverlayType = BrandingOverlayType.Branding10
}) => {
  var _overlay$metaOverlay10, _overlay$metaOverlay11;
  const [dimensions, setDimensions] = useState({
    cardWidth: 0,
    cardHeight: 0
  });
  const {
    i18n,
    t
  } = useTranslation();
  const {
    ColorPallet,
    TextTheme,
    ListItems
  } = useTheme();
  const [flaggedAttributes, setFlaggedAttributes] = useState();
  const [allPI, setAllPI] = useState();
  const credentialConnectionLabel = useCredentialConnectionLabel(credential);
  const [bundleResolver, credHelpActionOverrides] = useServices([TOKENS.UTIL_OCA_RESOLVER, TOKENS.CRED_HELP_ACTION_OVERRIDES]);
  const isBranding11 = brandingOverlayType === BrandingOverlayType.Branding11;
  const [helpAction, setHelpAction] = useState();
  const [overlay, setOverlay] = useState({});
  const {
    styles,
    borderRadius,
    logoHeight
  } = useCredentialCardStyles(overlay, brandingOverlayType, proof);
  const attributeFormats = useMemo(() => {
    var _overlay$bundle;
    return (_overlay$bundle = overlay.bundle) === null || _overlay$bundle === void 0 ? void 0 : _overlay$bundle.bundle.attributes.map(attr => {
      return {
        name: attr.name,
        format: attr.format
      };
    }).reduce((prev, curr) => {
      return {
        ...prev,
        [curr.name]: curr.format
      };
    }, {});
  }, [overlay]);
  const cardData = useMemo(() => {
    var _overlay$presentation, _overlay$presentation2;
    const fields = displayItems ?? [];
    const primaryField = overlay === null || overlay === void 0 || (_overlay$presentation = overlay.presentationFields) === null || _overlay$presentation === void 0 ? void 0 : _overlay$presentation.find(field => {
      var _overlay$brandingOver;
      return field.name === (overlay === null || overlay === void 0 || (_overlay$brandingOver = overlay.brandingOverlay) === null || _overlay$brandingOver === void 0 ? void 0 : _overlay$brandingOver.primaryAttribute);
    });
    const secondaryField = overlay === null || overlay === void 0 || (_overlay$presentation2 = overlay.presentationFields) === null || _overlay$presentation2 === void 0 ? void 0 : _overlay$presentation2.find(field => {
      var _overlay$brandingOver2;
      return field.name === (overlay === null || overlay === void 0 || (_overlay$brandingOver2 = overlay.brandingOverlay) === null || _overlay$brandingOver2 === void 0 ? void 0 : _overlay$brandingOver2.secondaryAttribute);
    });
    primaryField && fields.push(primaryField);
    secondaryField && fields.push(secondaryField);
    return fields;
  }, [displayItems, overlay]);
  const navigation = useNavigation();
  const backgroundColorIfRevoked = backgroundColor => credentialErrors.includes(CredentialErrors.Revoked) ? ColorPallet.notification.errorBorder : backgroundColor;
  const fontColorWithHighContrast = () => {
    var _overlay$brandingOver3;
    if (proof && brandingOverlayType === BrandingOverlayType.Branding10) {
      return ColorPallet.grayscale.mediumGrey;
    }
    const c = backgroundColorIfRevoked((_overlay$brandingOver3 = overlay.brandingOverlay) === null || _overlay$brandingOver3 === void 0 ? void 0 : _overlay$brandingOver3.primaryBackgroundColor) ?? ColorPallet.grayscale.lightGrey;
    const shade = shadeIsLightOrDark(c);
    return shade == Shade.Light ? ColorPallet.grayscale.darkGrey : ColorPallet.grayscale.lightGrey;
  };
  const parseAttribute = useCallback(item => {
    var _parsedItem, _parsedItem2, _parsedItem3;
    let parsedItem = item;
    if (item && item.pValue != null) {
      var _overlay$bundle2;
      parsedItem = pTypeToText(item, t, (_overlay$bundle2 = overlay.bundle) === null || _overlay$bundle2 === void 0 ? void 0 : _overlay$bundle2.captureBase.attributes);
    }
    const parsedValue = formatIfDate(attributeFormats === null || attributeFormats === void 0 ? void 0 : attributeFormats[item.name ?? ''], ((_parsedItem = parsedItem) === null || _parsedItem === void 0 ? void 0 : _parsedItem.value) ?? ((_parsedItem2 = parsedItem) === null || _parsedItem2 === void 0 ? void 0 : _parsedItem2.pValue) ?? null);
    return {
      label: item.label ?? item.name ?? '',
      value: item.value !== undefined && item.value != null ? parsedValue : `${(_parsedItem3 = parsedItem) === null || _parsedItem3 === void 0 ? void 0 : _parsedItem3.pType} ${parsedValue}`
    };
  }, [t, overlay, attributeFormats]);
  useEffect(() => {
    setAllPI(credential && cardData.every(item => {
      if (item === undefined) {
        return true;
      } else if (item instanceof Attribute) {
        const {
          label
        } = parseAttribute(item);
        return flaggedAttributes === null || flaggedAttributes === void 0 ? void 0 : flaggedAttributes.includes(label);
      } else {
        // Predicates are not PII
        return false;
      }
    }));
  }, [credential, cardData, parseAttribute, flaggedAttributes]);
  useEffect(() => {
    if (brandingOverlay) {
      setOverlay(brandingOverlay);
      return;
    }
    const params = {
      identifiers: credential ? getCredentialIdentifiers(credential) : {
        schemaId,
        credentialDefinitionId: credDefId
      },
      attributes: proof ? [] : credential === null || credential === void 0 ? void 0 : credential.credentialAttributes,
      meta: {
        credName,
        credConnectionId: credential === null || credential === void 0 ? void 0 : credential.connectionId,
        alias: credentialConnectionLabel
      },
      language: i18n.language
    };
    bundleResolver.resolveAllBundles(params).then(bundle => {
      if (proof) {
        var _Object$values;
        setFlaggedAttributes(bundle.bundle.bundle.flaggedAttributes.map(attr => attr.name));
        const credHelpUrl = bundle.bundle.bundle.metadata.credentialSupportUrl[params.language] ?? ((_Object$values = Object.values(bundle.bundle.bundle.metadata.credentialSupportUrl)) === null || _Object$values === void 0 ? void 0 : _Object$values[0]);

        // Check if there is a help action override for this credential
        const override = credHelpActionOverrides === null || credHelpActionOverrides === void 0 ? void 0 : credHelpActionOverrides.find(override => credDefId && override.credDefIds.includes(credDefId) || schemaId && override.schemaIds.includes(schemaId));
        if (override) {
          setHelpAction(() => () => {
            override.action(navigation);
          });
        } else if (credHelpUrl) {
          setHelpAction(() => () => {
            Linking.openURL(credHelpUrl);
          });
        }
      }
      setOverlay(o => ({
        ...o,
        ...bundle,
        brandingOverlay: bundle.brandingOverlay
      }));
    });
  }, [credential, schemaId, credDefId, credName, credentialConnectionLabel, i18n.language, bundleResolver, proof, credHelpActionOverrides, navigation, brandingOverlay]);
  const AttributeLabel = ({
    label
  }) => {
    var _overlay$bundle3;
    const ylabel = ((_overlay$bundle3 = overlay.bundle) === null || _overlay$bundle3 === void 0 || (_overlay$bundle3 = _overlay$bundle3.labelOverlay) === null || _overlay$bundle3 === void 0 ? void 0 : _overlay$bundle3.attributeLabels[label]) ?? startCase(label);
    return /*#__PURE__*/React.createElement(Text, {
      style: [TextTheme.labelSubtitle, styles.textContainer, {
        lineHeight: 19,
        opacity: 0.8
      }],
      testID: testIdWithKey('AttributeName')
    }, ylabel);
  };
  const AttributeErrorLabel = ({
    errorMessage
  }) => {
    return /*#__PURE__*/React.createElement(View, {
      style: {
        flexDirection: 'row',
        alignItems: 'center'
      }
    }, /*#__PURE__*/React.createElement(Icon, {
      testID: testIdWithKey('AttributeErrorIcon'),
      style: {
        paddingTop: 2,
        paddingHorizontal: 2
      },
      name: "close",
      color: ListItems.proofError.color,
      size: ListItems.recordAttributeText.fontSize
    }), /*#__PURE__*/React.createElement(Text, {
      testID: testIdWithKey('AttributeErrorText'),
      style: [TextTheme.labelSubtitle, styles.textContainer, {
        lineHeight: 19,
        opacity: 0.8,
        color: ListItems.proofError.color
      }]
    }, errorMessage));
  };
  const AttributeValue = ({
    value,
    warn
  }) => {
    return /*#__PURE__*/React.createElement(React.Fragment, null, isDataUrl(value) ? /*#__PURE__*/React.createElement(Image, {
      style: styles.imageAttr,
      source: {
        uri: value
      }
    }) : /*#__PURE__*/React.createElement(Text, {
      style: [TextTheme.bold, styles.textContainer, {
        color: warn ? ColorPallet.notification.warnText : styles.textContainer.color
      }],
      testID: testIdWithKey('AttributeValue')
    }, value));
  };
  const renderCardAttribute = item => {
    var _overlay$bundle4;
    const {
      label,
      value
    } = parseAttribute(item);
    const parsedValue = formatIfDate(item.format, value) ?? '';
    return /*#__PURE__*/React.createElement(View, {
      style: styles.cardAttributeContainer
    }, /*#__PURE__*/React.createElement(View, {
      style: {
        flexDirection: 'row',
        alignItems: 'center'
      }
    }, credentialErrors.includes(CredentialErrors.NotInWallet) && /*#__PURE__*/React.createElement(Icon, {
      testID: testIdWithKey('AttributeNameErrorIcon'),
      style: {
        paddingTop: 2,
        paddingHorizontal: 2
      },
      name: "close",
      color: ListItems.proofError.color,
      size: ListItems.recordAttributeText.fontSize
    }), /*#__PURE__*/React.createElement(AttributeLabel, {
      label: label
    })), !credentialErrors.includes(CredentialErrors.NotInWallet) && !item.hasError && /*#__PURE__*/React.createElement(View, {
      style: {
        flexDirection: 'row',
        alignItems: 'center'
      }
    }, (flaggedAttributes === null || flaggedAttributes === void 0 ? void 0 : flaggedAttributes.includes(label)) && !item.pValue && proof && /*#__PURE__*/React.createElement(Icon, {
      testID: testIdWithKey('AttributeNameWarningIcon'),
      style: {
        paddingTop: 2,
        paddingHorizontal: 2
      },
      name: "warning",
      color: ColorPallet.notification.warnIcon,
      size: ListItems.recordAttributeText.fontSize
    }), /*#__PURE__*/React.createElement(AttributeValue, {
      warn: (flaggedAttributes === null || flaggedAttributes === void 0 ? void 0 : flaggedAttributes.includes(label)) && !item.pValue && proof,
      value: parsedValue
    })), !credentialErrors.includes(CredentialErrors.NotInWallet) && item.hasError && /*#__PURE__*/React.createElement(AttributeErrorLabel, {
      errorMessage: t('ProofRequest.MissingAttribute', {
        name: ((_overlay$bundle4 = overlay.bundle) === null || _overlay$bundle4 === void 0 || (_overlay$bundle4 = _overlay$bundle4.labelOverlay) === null || _overlay$bundle4 === void 0 ? void 0 : _overlay$bundle4.attributeLabels[label]) ?? startCase(label)
      })
    }), !credentialErrors.includes(CredentialErrors.NotInWallet) && item.satisfied != undefined && item.satisfied === false && /*#__PURE__*/React.createElement(AttributeErrorLabel, {
      errorMessage: t('ProofRequest.PredicateNotSatisfied')
    }));
  };
  const CredentialCardPrimaryBodyFooter = () => {
    if (hasAltCredentials && handleAltCredChange) {
      var _overlay$brandingOver4;
      return /*#__PURE__*/React.createElement(CredentialActionFooter, {
        onPress: handleAltCredChange,
        text: t('ProofRequest.ChangeCredential'),
        testID: 'ChangeCredential',
        textColor: isBranding11 ? credentialTextColor(ColorPallet, (_overlay$brandingOver4 = overlay.brandingOverlay) === null || _overlay$brandingOver4 === void 0 ? void 0 : _overlay$brandingOver4.primaryBackgroundColor) : undefined
      });
    }
    if (Boolean(credentialErrors.length) && helpAction) {
      var _overlay$brandingOver5;
      return /*#__PURE__*/React.createElement(CredentialActionFooter, {
        onPress: helpAction,
        text: t('ProofRequest.GetThisCredential'),
        testID: 'GetThisCredential',
        textColor: isBranding11 ? credentialTextColor(ColorPallet, (_overlay$brandingOver5 = overlay.brandingOverlay) === null || _overlay$brandingOver5 === void 0 ? void 0 : _overlay$brandingOver5.primaryBackgroundColor) : undefined
      });
    }
    return /*#__PURE__*/React.createElement(React.Fragment, null);
  };
  const CredentialCardPrimaryBody = () => {
    var _overlay$metaOverlay, _overlay$metaOverlay2, _overlay$metaOverlay3;
    return /*#__PURE__*/React.createElement(View, {
      testID: testIdWithKey('CredentialCardPrimaryBody'),
      style: styles.primaryBodyContainer
    }, /*#__PURE__*/React.createElement(View, null, brandingOverlayType === BrandingOverlayType.Branding10 && /*#__PURE__*/React.createElement(CredentialIssuerBody, {
      overlay: overlay,
      overlayType: brandingOverlayType,
      elevated: elevated,
      proof: proof,
      hasBody: !(((_overlay$metaOverlay = overlay.metaOverlay) === null || _overlay$metaOverlay === void 0 ? void 0 : _overlay$metaOverlay.issuer) === 'Unknown Contact' && proof)
    }), /*#__PURE__*/React.createElement(View, {
      style: styles.primaryBodyNameContainer
    }, /*#__PURE__*/React.createElement(Text, {
      testID: testIdWithKey('CredentialName'),
      style: [TextTheme.bold, styles.textContainer, styles.credentialName, {
        color: allPI && proof ? ColorPallet.notification.warnText : styles.textContainer.color
      }]
    }, (_overlay$metaOverlay2 = overlay.metaOverlay) === null || _overlay$metaOverlay2 === void 0 ? void 0 : _overlay$metaOverlay2.name))), extraOverlayParameter && !displayItems && /*#__PURE__*/React.createElement(View, {
      style: {
        flex: 1,
        justifyContent: 'flex-end'
      }
    }, /*#__PURE__*/React.createElement(Text, {
      style: [TextTheme.caption, {
        color: styles.textContainer.color
      }]
    }, extraOverlayParameter.label ?? startCase(extraOverlayParameter.name || ''), ":", ' ', extraOverlayParameter.value)), /*#__PURE__*/React.createElement(View, {
      style: {
        flexDirection: 'row',
        alignItems: 'center'
      }
    }, credentialErrors.includes(CredentialErrors.Revoked) && Boolean(proof) && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Icon, {
      style: styles.errorIcon,
      name: "close",
      size: 30
    }), /*#__PURE__*/React.createElement(Text, {
      style: styles.errorText,
      testID: testIdWithKey('RevokedOrNotAvailable'),
      numberOfLines: 1
    }, t('CredentialDetails.Revoked'))), credentialErrors.includes(CredentialErrors.NotInWallet) && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Icon, {
      style: styles.errorIcon,
      name: "close",
      size: 30
    }), /*#__PURE__*/React.createElement(Text, {
      style: styles.errorText,
      testID: testIdWithKey('RevokedOrNotAvailable')
    }, t('ProofRequest.NotAvailableInYourWallet')))), /*#__PURE__*/React.createElement(FlatList, {
      data: cardData,
      scrollEnabled: false,
      initialNumToRender: cardData === null || cardData === void 0 ? void 0 : cardData.length,
      renderItem: ({
        item
      }) => {
        return renderCardAttribute(item);
      },
      ListFooterComponent: /*#__PURE__*/React.createElement(React.Fragment, null, brandingOverlayType === BrandingOverlayType.Branding11 && /*#__PURE__*/React.createElement(CredentialIssuerBody, {
        overlay: overlay,
        overlayType: brandingOverlayType,
        elevated: elevated,
        proof: proof,
        hasBody: !(((_overlay$metaOverlay3 = overlay.metaOverlay) === null || _overlay$metaOverlay3 === void 0 ? void 0 : _overlay$metaOverlay3.issuer) === 'Unknown Contact' && proof)
      }), /*#__PURE__*/React.createElement(CredentialCardPrimaryBodyFooter, null))
    }));
  };
  function getSliceBackgroundColor() {
    var _overlay$brandingOver6;
    if (hideSlice) return 'transparent';
    return brandingOverlayType === BrandingOverlayType.Branding10 ? backgroundColorIfRevoked(styles.secondaryBodyContainer.backgroundColor) : (_overlay$brandingOver6 = overlay.brandingOverlay) === null || _overlay$brandingOver6 === void 0 ? void 0 : _overlay$brandingOver6.secondaryBackgroundColor;
  }
  const CredentialCardSecondaryBody = () => {
    var _overlay$brandingOver7, _overlay$brandingOver8;
    return /*#__PURE__*/React.createElement(View, {
      testID: testIdWithKey('CredentialCardSecondaryBody'),
      style: [styles.secondaryBodyContainer, {
        backgroundColor: getSliceBackgroundColor() ?? ColorPallet.brand.secondaryBackground,
        overflow: 'hidden'
      }]
    }, (_overlay$brandingOver7 = overlay.brandingOverlay) !== null && _overlay$brandingOver7 !== void 0 && _overlay$brandingOver7.backgroundImageSlice && (!displayItems || brandingOverlayType === BrandingOverlayType.Branding11) && !hideSlice ? /*#__PURE__*/React.createElement(ImageBackground, {
      source: toImageSource((_overlay$brandingOver8 = overlay.brandingOverlay) === null || _overlay$brandingOver8 === void 0 ? void 0 : _overlay$brandingOver8.backgroundImageSlice),
      style: {
        flexGrow: 1
      },
      imageStyle: {
        borderTopLeftRadius: borderRadius,
        borderBottomLeftRadius: borderRadius
      }
    }) : !(Boolean(credentialErrors.length) || proof || getSecondaryBackgroundColor(overlay, proof)) && !hideSlice && /*#__PURE__*/React.createElement(View, {
      style: [{
        position: 'absolute',
        width: logoHeight,
        height: '100%',
        borderTopLeftRadius: borderRadius,
        borderBottomLeftRadius: borderRadius,
        backgroundColor: 'rgba(0,0,0,0.24)'
      }]
    }));
  };
  const CredentialCardStatus = ({
    status
  }) => {
    return /*#__PURE__*/React.createElement(View, {
      testID: testIdWithKey('CredentialCardStatus'),
      style: [styles.statusContainer, {
        position: 'absolute',
        right: 0,
        top: 0
      }]
    }, status ? /*#__PURE__*/React.createElement(View, {
      style: [styles.statusContainer, {
        backgroundColor: status === 'error' ? ColorPallet.notification.error : ColorPallet.notification.warn
      }]
    }, /*#__PURE__*/React.createElement(Icon, {
      size: 0.7 * logoHeight,
      style: {
        color: status === 'error' ? ColorPallet.semantic.error : ColorPallet.notification.warnIcon
      },
      name: status
    })) : /*#__PURE__*/React.createElement(View, {
      style: styles.statusContainer
    }));
  };
  const CredentialCard = ({
    status
  }) => {
    var _overlay$brandingOver9, _overlay$brandingOver10;
    const MainCredentialBody = () => {
      var _overlay$metaOverlay4, _overlay$metaOverlay5, _overlay$metaOverlay6, _overlay$metaOverlay7, _overlay$metaOverlay8, _overlay$metaOverlay9;
      return /*#__PURE__*/React.createElement(View, {
        style: styles.cardContainer,
        accessible: true,
        accessibilityLabel: `${(_overlay$metaOverlay4 = overlay.metaOverlay) !== null && _overlay$metaOverlay4 !== void 0 && _overlay$metaOverlay4.issuer ? `${t('Credentials.IssuedBy')} ${(_overlay$metaOverlay5 = overlay.metaOverlay) === null || _overlay$metaOverlay5 === void 0 ? void 0 : _overlay$metaOverlay5.issuer}` : ''}, ${((_overlay$metaOverlay6 = overlay.metaOverlay) === null || _overlay$metaOverlay6 === void 0 ? void 0 : _overlay$metaOverlay6.watermark) ?? ''} ${((_overlay$metaOverlay7 = overlay.metaOverlay) === null || _overlay$metaOverlay7 === void 0 ? void 0 : _overlay$metaOverlay7.name) ?? ''} ${t('Credentials.Credential')}.` + cardData.map(item => {
          const {
            label,
            value
          } = parseAttribute(item);
          if (label && value) {
            return ` ${label}, ${value}`;
          }
        })
      }, /*#__PURE__*/React.createElement(CredentialCardSecondaryBody, null), brandingOverlayType === BrandingOverlayType.Branding10 && /*#__PURE__*/React.createElement(CredentialCard11Logo, {
        noLogoText: ((_overlay$metaOverlay8 = overlay.metaOverlay) === null || _overlay$metaOverlay8 === void 0 ? void 0 : _overlay$metaOverlay8.name) ?? ((_overlay$metaOverlay9 = overlay.metaOverlay) === null || _overlay$metaOverlay9 === void 0 ? void 0 : _overlay$metaOverlay9.issuer) ?? 'C',
        overlay: overlay,
        overlayType: brandingOverlayType,
        elevated: elevated
      }), /*#__PURE__*/React.createElement(CredentialCardPrimaryBody, null), /*#__PURE__*/React.createElement(CredentialCardStatus, {
        status: status
      }));
    };
    return /*#__PURE__*/React.createElement(React.Fragment, null, (_overlay$brandingOver9 = overlay.brandingOverlay) !== null && _overlay$brandingOver9 !== void 0 && _overlay$brandingOver9.backgroundImage && hideSlice ? /*#__PURE__*/React.createElement(ImageBackground, {
      source: toImageSource((_overlay$brandingOver10 = overlay.brandingOverlay) === null || _overlay$brandingOver10 === void 0 ? void 0 : _overlay$brandingOver10.backgroundImage)
    }, /*#__PURE__*/React.createElement(MainCredentialBody, null)) : /*#__PURE__*/React.createElement(MainCredentialBody, null));
  };
  const getCredentialStatus = () => {
    if (brandingOverlayType === BrandingOverlayType.Branding10) {
      if (credentialErrors.includes(CredentialErrors.Revoked) && !proof) {
        return 'error';
      } else if (allPI && proof) {
        return 'warning';
      }
      return undefined;
    }
    if (credentialErrors.includes(CredentialErrors.Revoked)) {
      return 'error';
    } else if (allPI && proof) {
      return 'warning';
    }
    return undefined;
  };
  return overlay.bundle ? /*#__PURE__*/React.createElement(View, {
    style: [styles.container, style, {
      elevation: elevated ? 5 : 0,
      overflow: elevated ? 'visible' : 'hidden'
    }, hasAltCredentials ? styles.selectedCred : undefined],
    onLayout: event => {
      setDimensions({
        cardHeight: event.nativeEvent.layout.height,
        cardWidth: event.nativeEvent.layout.width
      });
    }
  }, /*#__PURE__*/React.createElement(TouchableOpacity, {
    accessible: false,
    accessibilityLabel: typeof onPress === 'undefined' ? undefined : t('Credentials.CredentialDetails'),
    disabled: typeof onPress === 'undefined' ? true : false,
    onPress: onPress,
    style: [styles.container, style],
    testID: testIdWithKey('ShowCredentialDetails')
  }, /*#__PURE__*/React.createElement(View, {
    testID: testIdWithKey('CredentialCard'),
    style: {
      overflow: 'hidden'
    }
  }, ((_overlay$metaOverlay10 = overlay.metaOverlay) === null || _overlay$metaOverlay10 === void 0 ? void 0 : _overlay$metaOverlay10.watermark) && /*#__PURE__*/React.createElement(CardWatermark, {
    width: dimensions.cardWidth,
    height: dimensions.cardHeight,
    style: {
      color: fontColorWithHighContrast()
    },
    watermark: (_overlay$metaOverlay11 = overlay.metaOverlay) === null || _overlay$metaOverlay11 === void 0 ? void 0 : _overlay$metaOverlay11.watermark
  }), /*#__PURE__*/React.createElement(CredentialCard
  // if the credential is revoked and we are not in a proof context
  // display a small error icon in the top right of the credential card
  , {
    status: getCredentialStatus()
  })))) : null;
};
export default CredentialCard11;
//# sourceMappingURL=CredentialCard11.js.map